@@ -31,6 +31,10 @@ pub struct CollectionBuilder {
     pub page_size: usize,
     /// A human readable name
     pub name: Option<String>,
+    /// Whether it's children should be included as nested resources in the response
+    pub include_nested: bool,
+    /// Whether to include resources from other servers
+    pub include_external: bool,
 }
 
 impl CollectionBuilder {
@@ -52,6 +56,20 @@ impl CollectionBuilder {
         if let Some(val) = &self.sort_by {
             resource.set_propval_string(crate::urls::COLLECTION_SORT_BY.into(), val, store)?;
         }
+        if self.include_nested {
+            resource.set_propval_string(
+                crate::urls::COLLECTION_INCLUDE_NESTED.into(),
+                "true",
+                store,
+            )?;
+        }
+        if self.include_external {
+            resource.set_propval_string(
+                crate::urls::COLLECTION_INCLUDE_EXTERNAL.into(),
+                "true",
+                store,
+            )?;
+        }
         if self.sort_desc {
             resource.set_propval_string(crate::urls::COLLECTION_SORT_DESC.into(), "true", store)?;
         }
@@ -62,7 +80,7 @@ impl CollectionBuilder {
         )?;
         resource.set_propval(
             crate::urls::COLLECTION_PAGE_SIZE.into(),
-            self.page_size.clone().into(),
+            self.page_size.into(),
             store,
         )?;
         // Maybe include items directly
@@ -84,6 +102,8 @@ impl CollectionBuilder {
             page_size: DEFAULT_PAGE_SIZE,
             current_page: 0,
             name: Some(format!("{} collection", path)),
+            include_nested: true,
+            include_external: false,
         }
     }
 
@@ -106,6 +126,8 @@ pub struct Collection {
     pub value: Option<String>,
     /// The actual items that you're interested in. List the member subjects of the current page.
     pub members: Vec<String>,
+    /// The members as full resources, instead of a list of subjects. Is only populated if `nested` is true.
+    pub members_nested: Option<Vec<Resource>>,
     /// URL of the value to sort by
     pub sort_by: Option<String>,
     // Sorts ascending by default
@@ -120,6 +142,10 @@ pub struct Collection {
     pub total_pages: usize,
     /// Human readable name of a resource
     pub name: Option<String>,
+    /// Whether it's children should be included as nested resources in the response
+    pub include_nested: bool,
+    /// Include resources from other servers
+    pub include_external: bool,
 }
 
 /// Sorts a vector or resources by some property.
@@ -165,36 +191,47 @@ impl Collection {
             None,
             collection_builder.property.as_deref(),
             collection_builder.value.as_deref(),
-            // Collections only show items from inside this store. Maybe later add this as an option to collections
-            false,
+            collection_builder.include_external,
         )?;
-        let mut subjects: Vec<String> = atoms.iter().map(|atom| atom.subject.clone()).collect();
-        // Default to no sorting
-        if collection_builder.sort_by.is_some() {
-            let mut resources = Vec::new();
-            for subject in subjects.clone() {
-                resources.push(store.get_resource(&subject)?)
+        // Remove duplicate subjects
+        let mut subjects: Vec<String> = atoms
+            .iter()
+            .map(|atom| atom.subject.clone())
+            .collect::<std::collections::HashSet<String>>()
+            .into_iter()
+            .collect();
+
+        let mut resources = Vec::new();
+        // If sorting is required or the nested resoureces are asked, we need to fetch all resources from the store.
+        if collection_builder.sort_by.is_some() || collection_builder.include_nested {
+            for subject in subjects.iter() {
+                // These nested resources are not fully calculated - they will be presented as -is
+                let resource = store.get_resource_extended(subject, true)?;
+                resources.push(resource)
             }
-            // TODO: Include these resources in the response! They're already fetched. Should speed things up.
-            // https://github.com/joepio/atomic/issues/62
-            resources = sort_resources(
-                resources,
-                &collection_builder.sort_by.clone().unwrap(),
-                collection_builder.sort_desc,
-            );
-            subjects.clear();
-            for resource in resources {
-                subjects.push(resource.get_subject().clone())
+            if let Some(sort) = &collection_builder.sort_by {
+                resources = sort_resources(resources, sort, collection_builder.sort_desc);
+                subjects.clear();
+                for r in resources.iter() {
+                    subjects.push(r.get_subject().clone())
+                }
             }
         }
         let mut all_pages: Vec<Vec<String>> = Vec::new();
+        let mut all_pages_nested: Vec<Vec<Resource>> = Vec::new();
         let mut page: Vec<String> = Vec::new();
+        let mut page_nested: Vec<Resource> = Vec::new();
         let current_page = collection_builder.current_page;
         for (i, subject) in subjects.iter().enumerate() {
             page.push(subject.into());
+            if collection_builder.include_nested {
+                page_nested.push(resources[i].clone());
+            }
             if page.len() >= collection_builder.page_size {
                 all_pages.push(page);
+                all_pages_nested.push(page_nested);
                 page = Vec::new();
+                page_nested = Vec::new();
                 // No need to calculte more than necessary
                 if all_pages.len() > current_page {
                     break;
@@ -203,24 +240,37 @@ impl Collection {
             // Add the last page when handling the last subject
             if i == subjects.len() - 1 {
                 all_pages.push(page);
+                all_pages_nested.push(page_nested);
                 break;
             }
         }
         if all_pages.is_empty() {
-            all_pages.push(Vec::new())
+            all_pages.push(Vec::new());
+            all_pages_nested.push(Vec::new());
         }
         // Maybe I should default to last page, if current_page is too high?
         let members = all_pages
             .get(current_page)
-            .ok_or("Page number is too high")?
+            .ok_or(format!("Page number {} is too high", current_page))?
             .clone();
         let total_items = subjects.len();
         // Construct the pages (TODO), use pageSize
         let total_pages =
             (total_items + collection_builder.page_size - 1) / collection_builder.page_size;
+        let members_nested = if collection_builder.include_nested {
+            Some(
+                all_pages_nested
+                    .get(current_page)
+                    .ok_or(format!("Page number {} is too high", current_page))?
+                    .clone(),
+            )
+        } else {
+            None
+        };
         let collection = Collection {
             total_pages,
             members,
+            members_nested,
             total_items,
             subject: collection_builder.subject,
             property: collection_builder.property,
@@ -230,6 +280,8 @@ impl Collection {
             current_page: collection_builder.current_page,
             page_size: collection_builder.page_size,
             name: collection_builder.name,
+            include_nested: collection_builder.include_nested,
+            include_external: collection_builder.include_external,
         };
         Ok(collection)
     }
@@ -248,12 +300,30 @@ impl Collection {
     ) -> AtomicResult<crate::Resource> {
         resource.set_propval(
             crate::urls::COLLECTION_MEMBERS.into(),
-            self.members.clone().into(),
+            if let Some(nested_members) = &self.members_nested {
+                nested_members.clone().into()
+            } else {
+                self.members.clone().into()
+            },
             store,
         )?;
         if let Some(prop) = &self.property {
             resource.set_propval_string(crate::urls::COLLECTION_PROPERTY.into(), prop, store)?;
         }
+        if self.include_nested {
+            resource.set_propval_string(
+                crate::urls::COLLECTION_INCLUDE_NESTED.into(),
+                "true",
+                store,
+            )?;
+        }
+        if self.include_external {
+            resource.set_propval_string(
+                crate::urls::COLLECTION_INCLUDE_EXTERNAL.into(),
+                "true",
+                store,
+            )?;
+        }
         if let Some(val) = &self.value {
             resource.set_propval_string(crate::urls::COLLECTION_VALUE.into(), val, store)?;
         }
@@ -262,32 +332,34 @@ impl Collection {
         }
         resource.set_propval(
             crate::urls::COLLECTION_MEMBER_COUNT.into(),
-            self.total_items.clone().into(),
+            self.total_items.into(),
             store,
         )?;
         let classes: Vec<String> = vec![crate::urls::COLLECTION.into()];
         resource.set_propval(crate::urls::IS_A.into(), classes.into(), store)?;
         resource.set_propval(
             crate::urls::COLLECTION_TOTAL_PAGES.into(),
-            self.total_pages.clone().into(),
+            self.total_pages.into(),
             store,
         )?;
         resource.set_propval(
             crate::urls::COLLECTION_CURRENT_PAGE.into(),
-            self.current_page.clone().into(),
+            self.current_page.into(),
             store,
         )?;
         resource.set_propval(
             crate::urls::COLLECTION_PAGE_SIZE.into(),
-            self.page_size.clone().into(),
+            self.page_size.into(),
             store,
         )?;
-        // Maybe include items directly
+
         Ok(resource.to_owned())
     }
 }
 
-/// Builds a collection from query params
+/// Builds a collection from query params and the passed Collection resource.
+/// The query params are used to override the stored Collection resource properties.
+/// This also sets defaults for Collection properties when fields are missing
 pub fn construct_collection(
     store: &impl Storelike,
     query_params: url::form_urlencoded::Parse,
@@ -300,6 +372,8 @@ pub fn construct_collection(
     let mut value = None;
     let mut property = None;
     let mut name = None;
+    let mut include_nested = false;
+    let mut include_external = false;
 
     if let Ok(val) = resource.get(urls::COLLECTION_PROPERTY) {
         property = Some(val.to_string());
@@ -310,15 +384,22 @@ pub fn construct_collection(
     if let Ok(val) = resource.get(urls::NAME) {
         name = Some(val.to_string());
     }
+    if let Ok(val) = resource.get(urls::COLLECTION_INCLUDE_NESTED) {
+        include_nested = val.to_bool()?;
+    }
+    if let Ok(val) = resource.get(urls::COLLECTION_INCLUDE_EXTERNAL) {
+        include_external = val.to_bool()?;
+    }
     for (k, v) in query_params {
         match k.as_ref() {
             "property" => property = Some(v.to_string()),
             "value" => value = Some(v.to_string()),
             "sort_by" => sort_by = Some(v.to_string()),
-            // TODO: parse bool
-            "sort_desc" => sort_desc = true,
+            "sort_desc" => sort_desc = v.parse::<bool>()?,
             "current_page" => current_page = v.parse::<usize>()?,
             "page_size" => page_size = v.parse::<usize>()?,
+            "include_nested" => include_nested = v.parse::<bool>()?,
+            "include_external" => include_external = v.parse::<bool>()?,
             _ => {}
         };
     }
@@ -331,6 +412,8 @@ pub fn construct_collection(
         current_page,
         page_size,
         name,
+        include_nested,
+        include_external,
     };
     let collection = Collection::new_with_members(store, collection_builder)?;
     collection.add_to_resource(resource, store)
@@ -346,7 +429,6 @@ mod test {
     fn create_collection() {
         let store = crate::Store::init().unwrap();
         store.populate().unwrap();
-        // Get all Classes, sorted by shortname
         let collection_builder = CollectionBuilder {
             subject: "test_subject".into(),
             property: Some(urls::IS_A.into()),
@@ -356,6 +438,8 @@ mod test {
             page_size: DEFAULT_PAGE_SIZE,
             current_page: 0,
             name: Some("Test collection".into()),
+            include_nested: false,
+            include_external: false,
         };
         let collection = Collection::new_with_members(&store, collection_builder).unwrap();
         assert!(collection.members.contains(&urls::PROPERTY.into()));
@@ -365,7 +449,6 @@ mod test {
     fn create_collection_2() {
         let store = crate::Store::init().unwrap();
         store.populate().unwrap();
-        // Get all Classes, sorted by shortname
         let collection_builder = CollectionBuilder {
             subject: "test_subject".into(),
             property: Some(urls::IS_A.into()),
@@ -375,9 +458,46 @@ mod test {
             page_size: DEFAULT_PAGE_SIZE,
             current_page: 0,
             name: None,
+            include_nested: false,
+            include_external: false,
         };
         let collection = Collection::new_with_members(&store, collection_builder).unwrap();
         assert!(collection.members.contains(&urls::PROPERTY.into()));
+
+        let resource_collection = &collection.to_resource(&store).unwrap();
+        resource_collection
+            .get(urls::COLLECTION_INCLUDE_NESTED)
+            .unwrap_err();
+    }
+
+    #[test]
+    fn create_collection_nested_members_and_sorting() {
+        let store = crate::Store::init().unwrap();
+        store.populate().unwrap();
+        let collection_builder = CollectionBuilder {
+            subject: "test_subject".into(),
+            property: Some(urls::IS_A.into()),
+            value: Some(urls::CLASS.into()),
+            sort_by: Some(urls::SHORTNAME.into()),
+            sort_desc: false,
+            page_size: DEFAULT_PAGE_SIZE,
+            current_page: 0,
+            name: None,
+            // The important bit here
+            include_nested: true,
+            include_external: false,
+        };
+        let collection = Collection::new_with_members(&store, collection_builder).unwrap();
+        let first_resource = &collection.members_nested.clone().unwrap()[0];
+        assert!(first_resource.get_subject().contains("Agent"));
+
+        let resource_collection = &collection.to_resource(&store).unwrap();
+        let val = resource_collection
+            .get(urls::COLLECTION_INCLUDE_NESTED)
+            .unwrap()
+            .to_bool()
+            .unwrap();
+        assert!(val, "Include nested must be true");
     }
 
     #[cfg(feature = "db")]
@@ -391,7 +511,7 @@ mod test {
             .collect();
         println!("{:?}", subjects);
         let collections_collection = store
-            .get_resource_extended(&format!("{}/collections", store.get_base_url()))
+            .get_resource_extended(&format!("{}/collections", store.get_base_url()), false)
             .unwrap();
         assert!(
             collections_collection
@@ -416,7 +536,7 @@ mod test {
         store.populate().unwrap();
 
         let collection_page_size = store
-            .get_resource_extended("https://atomicdata.dev/classes?page_size=1")
+            .get_resource_extended("https://atomicdata.dev/classes?page_size=1", false)
             .unwrap();
         assert!(
             collection_page_size
@@ -426,7 +546,10 @@ mod test {
                 == "1"
         );
         let collection_page_nr = store
-            .get_resource_extended("https://atomicdata.dev/classes?current_page=2&page_size=1")
+            .get_resource_extended(
+                "https://atomicdata.dev/classes?current_page=2&page_size=1",
+                false,
+            )
             .unwrap();
         assert!(
             collection_page_nr
@@ -436,7 +559,7 @@ mod test {
                 == "1"
         );
         let members_vec = match collection_page_nr.get(urls::COLLECTION_MEMBERS).unwrap() {
-            crate::Value::ResourceArray(vec) => vec,
+            crate::Value::ResourceArraySubjects(vec) => vec,
             _ => panic!(),
         };
         assert!(members_vec.len() == 1);
