@@ -97,7 +97,7 @@ impl Db {
                 let psm: PropSubjectMap = bincode::deserialize(binpropval).map_err(|e| {
                     format!(
                         "Deserialize PropSubjectMap error: {} {}",
-                        corrupt_db_message(&string_val),
+                        corrupt_db_message(string_val),
                         e
                     )
                 })?;
@@ -161,7 +161,7 @@ impl Storelike for Db {
     // This only adds ResourceArrays and AtomicURLs at this moment, which means that many values cannot be accessed in the TPF query (thus, collections)
     fn add_atom_to_index(&self, atom: &Atom) -> AtomicResult<()> {
         let vec = match atom.value.clone() {
-            Value::ResourceArray(v) => v,
+            Value::ResourceArraySubjects(v) => v,
             Value::AtomicUrl(v) => vec![v],
             _other => return Ok(()),
         };
@@ -186,16 +186,16 @@ impl Storelike for Db {
         // This only works if no external functions rely on using add_resource for atom-like operations!
         // However, add_atom uses set_propvals, which skips the validation.
         resource.check_required_props(self)?;
-        self.set_propvals(resource.get_subject(), &resource.get_propvals())
+        self.set_propvals(resource.get_subject(), resource.get_propvals())
     }
 
     fn add_resource_unsafe(&self, resource: &Resource) -> AtomicResult<()> {
-        self.set_propvals(resource.get_subject(), &resource.get_propvals())
+        self.set_propvals(resource.get_subject(), resource.get_propvals())
     }
 
     fn remove_atom_from_index(&self, atom: &Atom) -> AtomicResult<()> {
         let vec = match atom.value.clone() {
-            Value::ResourceArray(v) => v,
+            Value::ResourceArraySubjects(v) => v,
             other => vec![other.to_string()],
         };
 
@@ -244,7 +244,7 @@ impl Storelike for Db {
         }
     }
 
-    fn get_resource_extended(&self, subject: &str) -> AtomicResult<Resource> {
+    fn get_resource_extended(&self, subject: &str, skip_dynamic: bool) -> AtomicResult<Resource> {
         // This might add a trailing slash
         let mut url = url::Url::parse(subject)?;
         let clone = url.clone();
@@ -275,30 +275,54 @@ impl Storelike for Db {
         }
 
         let mut resource = self.get_resource(&removed_query_params)?;
+
         // make sure the actual subject matches the one requested
         resource.set_subject(subject.into());
+
+        // Whether the resource has dynamic properties
+        let mut has_dynamic = false;
         // If a certain class needs to be extended, add it to this match statement
         for class in resource.get_classes(self)? {
             match class.subject.as_ref() {
                 crate::urls::COLLECTION => {
-                    return crate::collections::construct_collection(
-                        self,
-                        query_params,
-                        &mut resource,
-                    )
+                    has_dynamic = true;
+                    if !skip_dynamic {
+                        return crate::collections::construct_collection(
+                            self,
+                            query_params,
+                            &mut resource,
+                        );
+                    }
                 }
                 crate::urls::INVITE => {
-                    return crate::plugins::invite::construct_invite_redirect(
-                        self,
-                        query_params,
-                        &mut resource,
-                        subject,
-                    )
+                    has_dynamic = true;
+                    if !skip_dynamic {
+                        return crate::plugins::invite::construct_invite_redirect(
+                            self,
+                            query_params,
+                            &mut resource,
+                            subject,
+                        );
+                    }
+                }
+                crate::urls::DRIVE => {
+                    has_dynamic = true;
+                    if !skip_dynamic {
+                        return crate::hierarchy::add_children(self, &mut resource);
+                    }
                 }
-                crate::urls::DRIVE => return crate::hierarchy::add_children(self, &mut resource),
                 _ => {}
             }
         }
+
+        // This lets clients know that the resource may have dynamic properties that are currently not included
+        if has_dynamic && skip_dynamic {
+            resource.set_propval(
+                crate::urls::INCOMPLETE.into(),
+                crate::Value::Boolean(true),
+                self,
+            )?;
+        }
         Ok(resource)
     }
 
@@ -415,7 +439,7 @@ impl Storelike for Db {
         };
 
         match q_subject {
-            Some(sub) => match self.get_resource(&sub) {
+            Some(sub) => match self.get_resource(sub) {
                 Ok(resource) => {
                     if hasprop | hasval {
                         find_in_resource(&resource);
@@ -564,14 +588,21 @@ pub mod test {
             .collect();
         println!("{:?}", subjects);
         let collections_collection_url = format!("{}/collections", store.get_base_url());
-        let my_resource = store
-            .get_resource_extended(&collections_collection_url)
+        let collections_resource = store
+            .get_resource_extended(&collections_collection_url, false)
             .unwrap();
-        let my_value = my_resource
+        let member_count = collections_resource
             .get(crate::urls::COLLECTION_MEMBER_COUNT)
+            .unwrap()
+            .to_int()
             .unwrap();
-        println!("My value: {}", my_value);
-        assert!(my_value.to_int().unwrap() > 11);
+        assert!(member_count > 11);
+        let nested = collections_resource
+            .get(crate::urls::COLLECTION_INCLUDE_NESTED)
+            .unwrap()
+            .to_bool()
+            .unwrap();
+        assert!(nested);
     }
 
     #[test]
@@ -602,7 +633,7 @@ pub mod test {
     fn destroy_resource_and_check_collection() {
         let store = DB.lock().unwrap().clone();
         let agents_url = format!("{}/agents", store.get_base_url());
-        let agents_collection_1 = store.get_resource_extended(&agents_url).unwrap();
+        let agents_collection_1 = store.get_resource_extended(&agents_url, false).unwrap();
         let agents_collection_count_1 = agents_collection_1
             .get(crate::urls::COLLECTION_MEMBER_COUNT)
             .unwrap()
@@ -618,7 +649,7 @@ pub mod test {
             .to_resource(&store)
             .unwrap();
         resource.save_locally(&store).unwrap();
-        let agents_collection_2 = store.get_resource_extended(&agents_url).unwrap();
+        let agents_collection_2 = store.get_resource_extended(&agents_url, false).unwrap();
         let agents_collection_count_2 = agents_collection_2
             .get(crate::urls::COLLECTION_MEMBER_COUNT)
             .unwrap()
@@ -630,7 +661,7 @@ pub mod test {
         );
 
         resource.destroy(&store).unwrap();
-        let agents_collection_3 = store.get_resource_extended(&agents_url).unwrap();
+        let agents_collection_3 = store.get_resource_extended(&agents_url, false).unwrap();
         let agents_collection_count_3 = agents_collection_3
             .get(crate::urls::COLLECTION_MEMBER_COUNT)
             .unwrap()
