@@ -7,13 +7,13 @@ use crate::{
     schema::{Class, Property},
     Atom, Storelike,
 };
+use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 
 /// A Resource is a set of Atoms that shares a single Subject.
 /// A Resource only contains valid Values, but it _might_ lack required properties.
 /// All changes to the Resource are applied after committing them (e.g. by using).
-// #[derive(Clone, Debug, Serialize, Deserialize)]
-#[derive(Clone, Debug)]
+#[derive(Clone, Serialize, Deserialize, Debug)]
 pub struct Resource {
     /// A hashMap of all the Property Value combinations
     propvals: PropVals,
@@ -73,12 +73,21 @@ impl Resource {
         let mut classes: Vec<Class> = Vec::new();
         if let Ok(val) = self.get(crate::urls::IS_A) {
             for class in val.to_vec()? {
-                classes.push(store.get_class(&class)?)
+                classes.push(store.get_class(class)?)
             }
         }
         Ok(classes)
     }
 
+    /// Returns the first item of the is_ array
+    pub fn get_main_class(&self) -> AtomicResult<String> {
+        if let Ok(val) = self.get(crate::urls::IS_A) {
+            Ok(val.to_vec()?[0].clone())
+        } else {
+            Err(format!("Resource {} has no class", self.subject).into())
+        }
+    }
+
     /// Returns all PropVals.
     /// Useful if you want to iterate over all Atoms / Properties.
     pub fn get_propvals(&self) -> &PropVals {
